SystemOrganization addCategory: #'PackageInfo-Base'!

!MethodReference methodsFor: '*PackageInfo-Base' stamp: 'ab 5/23/2003 22:58'!
sourceCode
	^ self actualClass sourceCodeAt: methodSymbol! !

Object subclass: #PackageInfo
	instanceVariableNames: 'packageName methodCategoryPrefix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PackageInfo-Base'!

!PackageInfo commentStamp: '<historical>' prior: 0!
Subclass this class to create new Packages.!

!PackageInfo class methodsFor: 'packages access' stamp: 'Alexandre.Bergel 4/4/2009 09:26'!
allPackages
	^PackageOrganizer default packages! !

!PackageInfo class methodsFor: 'compatibility' stamp: 'avi 3/9/2004 16:28'!
default
	^ self allPackages detect: [:ea | ea class = self] ifNone: [self new register]! !

!PackageInfo class methodsFor: 'testing' stamp: 'Alexandre.Bergel 4/4/2009 09:29'!
existPackageNamed: aString
	"
	self existPackageNamed: 'PackageInfo'
	self existPackageNamed: 'Zork'
	"
	^ (self allPackages anySatisfy: [:each | each packageName = aString])
			! !

!PackageInfo class methodsFor: 'initialization' stamp: 'avi 2/18/2004 00:46'!
initialize
	self allSubclassesDo: [:ea | ea new register]! !

!PackageInfo class methodsFor: 'packages access' stamp: 'avi 11/12/2003 23:00'!
named: aString
	^ PackageOrganizer default packageNamed: aString ifAbsent: [(self new packageName: aString) register]! !

!PackageInfo class methodsFor: 'packages access' stamp: 'avi 11/11/2003 17:19'!
registerPackageName: aString
	^ PackageOrganizer default registerPackageNamed: aString! !

!PackageInfo methodsFor: 'comparing' stamp: 'avi 10/11/2003 00:09'!
= other
	^ other species = self species and: [other packageName = self packageName]! !

!PackageInfo methodsFor: 'enumerating' stamp: 'eem 8/30/2010 10:28'!
actualMethodsDo: aBlock
	"Evaluate aBlock with the actual method objects in this package."
	| enum |
	self extensionMethods do:
		[:mr|
		aBlock value: mr compiledMethod].
	enum := [:behavior|
			behavior organization categories do:
				[:cat|
				(self isForeignClassExtension: cat) ifFalse:
					[(behavior organization listAtCategoryNamed: cat) do:
						[:s|
						aBlock value: (behavior compiledMethodAt: s)]]]].
	self classes do:
		[:c| enum value: c; value: c classSide]
		! !

!PackageInfo methodsFor: 'modifying' stamp: 'stephaneducasse 2/4/2006 20:40'!
addCoreMethod: aMethodReference
	| category |
	category := self baseCategoryOfMethod: aMethodReference.
	aMethodReference actualClass organization
		classify: aMethodReference methodSymbol
		under: category
		suppressIfDefault: false! !

!PackageInfo methodsFor: 'modifying' stamp: 'stephaneducasse 2/4/2006 20:40'!
addExtensionMethod: aMethodReference
	| category |
	category := self baseCategoryOfMethod: aMethodReference.
	aMethodReference actualClass organization
		classify: aMethodReference methodSymbol
		under: self methodCategoryPrefix, '-', category! !

!PackageInfo methodsFor: 'modifying' stamp: 'StephaneDucasse 8/18/2009 23:17'!
addMethod: aMethodReference
	(self includesClass: aMethodReference actualClass)
		ifTrue: [self addCoreMethod: aMethodReference]
		ifFalse: [self addExtensionMethod: aMethodReference]! !

!PackageInfo methodsFor: 'listing' stamp: 'StephaneDucasse 3/3/2010 13:47'!
allOverriddenMethods
	"search classes and meta classes"
	^ Array streamContents: [:stream |
		self allOverriddenMethodsDo: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'StephaneDucasse 3/3/2010 13:46'!
allOverriddenMethodsDo: aBlock
	"Evaluates aBlock with all the overridden methods in the system"
	^ ProtoObject withAllSubclassesDo: [:class | 
		self overriddenMethodsInClass: class do: aBlock]
! !

!PackageInfo methodsFor: 'modifying' stamp: 'stephaneducasse 2/4/2006 20:40'!
baseCategoryOfMethod: aMethodReference
	| oldCat oldPrefix tokens | 
	oldCat := aMethodReference category.
	({ 'as yet unclassified'. 'all' } includes: oldCat) ifTrue: [ oldCat := '' ].
	tokens := oldCat findTokens: '*-' keep: '*'.

	"Strip off any old prefixes"
	((tokens at: 1 ifAbsent: [ '' ]) = '*') ifTrue: [
		[ ((tokens at: 1 ifAbsent: [ '' ]) = '*') ]
			whileTrue: [ tokens removeFirst ].
		oldPrefix := tokens removeFirst asLowercase.
		[ (tokens at: 1 ifAbsent: [ '' ]) asLowercase = oldPrefix ]
			whileTrue: [ tokens removeFirst ].
	].

	tokens isEmpty ifTrue: [^ 'as yet unclassified'].
	^ String streamContents:
		[ :s |
		tokens
			do: [ :tok | s nextPutAll: tok ]
			separatedBy: [ s nextPut: $- ]]! !

!PackageInfo methodsFor: 'testing' stamp: 'StephaneDucasse 2/6/2010 17:40'!
category: categoryName matches: prefix

	| prefixSize catSize |
	categoryName ifNil: [ ^false ].
	catSize := categoryName size.
	prefixSize := prefix size.
	catSize < prefixSize ifTrue: [ ^false ].
	(categoryName findString: prefix startingAt: 1 caseSensitive: false) = 1
		ifFalse: [ ^false ].
	^(categoryName at: prefix size + 1 ifAbsent: [ ^true ]) = $-! !

!PackageInfo methodsFor: 'naming' stamp: 'stephaneducasse 2/4/2006 20:40'!
categoryName
	|category|
	category := self class category.
	^ (category endsWith: '-Info')
		ifTrue: [category copyUpToLast: $-]
		ifFalse: [category]! !

!PackageInfo methodsFor: 'testing' stamp: 'StephaneDucasse 5/2/2010 20:21'!
changeRecordForOverriddenMethod: aMethodReference
	| sourceFilesCopy method position |
	method := aMethodReference actualClass compiledMethodAt: aMethodReference methodSymbol.
	position := method filePosition.
	sourceFilesCopy := SourceFiles
		collect: [ :x | 
			x isNil
				ifTrue: [ nil ]
				ifFalse: [ x readOnlyCopy ] ].
	[ 
	| file prevPos prevFileIndex chunk stamp methodCategory tokens |
	method fileIndex == 0
		ifTrue: [ ^ nil ].
	file := sourceFilesCopy at: method fileIndex.
	[ position notNil & file notNil ]
		whileTrue: [ 
			file position: (0 max: position - 150).	"Skip back to before the preamble"
			[ file position < (position - 1) ] whileTrue: [ chunk := file nextChunk ].	"then pick it up from the front"	"Preamble is likely a linked method preamble, if we're in
			a changes file (not the sources file).  Try to parse it
			for prior source position and file index"
			prevPos := nil.
			stamp := ''.
			(chunk findString: 'methodsFor:' startingAt: 1) > 0
				ifTrue: [ tokens := Scanner new scanTokens: chunk ]
				ifFalse: [ tokens := Array new	"ie cant be back ref" ].
			((tokens size between: 7 and: 8) and: [ (tokens at: tokens size - 5) = #methodsFor: ])
				ifTrue: [ 
					(tokens at: tokens size - 3) = #stamp:
						ifTrue: [ 
							"New format gives change stamp and unified prior pointer"
							stamp := tokens at: tokens size - 2.
							prevPos := tokens last.
							prevFileIndex := sourceFilesCopy fileIndexFromSourcePointer: prevPos.
							prevPos := sourceFilesCopy filePositionFromSourcePointer: prevPos ]
						ifFalse: [ 
							"Old format gives no stamp; prior pointer in two parts"
							prevPos := tokens at: tokens size - 2.
							prevFileIndex := tokens last ].
					(prevPos = 0 or: [ prevFileIndex = 0 ])
						ifTrue: [ prevPos := nil ] ].
			((tokens size between: 5 and: 6) and: [ (tokens at: tokens size - 3) = #methodsFor: ])
				ifTrue: [ 
					(tokens at: tokens size - 1) = #stamp:
						ifTrue: [ 
							"New format gives change stamp and unified prior pointer"
							stamp := tokens at: tokens size ] ].
			methodCategory := tokens after: #methodsFor: ifAbsent: [ 'as yet unclassifed' ].
			(self includesMethodCategory: methodCategory ofClass: aMethodReference actualClass)
				ifTrue: [ 
					methodCategory = (Smalltalk globals at: #Categorizer ifAbsent: [ Smalltalk globals at: #ClassOrganizer ]) default
						ifTrue: [ methodCategory := methodCategory , ' ' ].
					^ ChangeRecord new
						file: file
						position: position
						type: #method
						class: aMethodReference classSymbol
						category: methodCategory
						meta: aMethodReference classIsMeta
						stamp: stamp ].
			position := prevPos.
			prevPos notNil
				ifTrue: [ file := sourceFilesCopy at: prevFileIndex ] ].
	^ nil ]
		ensure: [ 
			sourceFilesCopy
				do: [ :x | 
					x notNil
						ifTrue: [ x close ] ] ]! !

!PackageInfo methodsFor: 'accessing' stamp: 'sd 9/12/2010 19:07'!
classes
	^ (self systemCategories
		gather: [ :cat | (self systemOrganization listAtCategoryNamed: cat) collect: [ :className | self environment at: className ] ])
		sortBy: [ :a :b | a className <= b className ] ! !

!PackageInfo methodsFor: 'listing' stamp: 'ul 11/13/2009 16:29'!
classesAndMetaClasses
	"Return a Set with all classes and metaclasses belonging to this package"

	| baseClasses result |
	baseClasses := self classes.
	result := (Set new: baseClasses size * 2) 
		addAll: baseClasses;
		yourself.
	baseClasses do: [ :c | 
		result add: c classSide].
	^result
! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:18'!
coreCategoriesForClass: aClass
	^ aClass organization categories select: [:cat | (self isForeignClassExtension: cat) not]! !

!PackageInfo methodsFor: 'listing' stamp: 'ab 11/13/2002 01:23'!
coreMethods
	^ self classesAndMetaClasses gather: [:class | self coreMethodsForClass: class]! !

!PackageInfo methodsFor: 'testing' stamp: 'nice 12/4/2009 00:02'!
coreMethodsForClass: aClass
	^ (aClass selectors asSet difference:
		((self foreignExtensionMethodsForClass: aClass) collect: [:r | r methodSymbol]))
			asArray collect: [:sel | self referenceForMethod: sel ofClass: aClass]! !

!PackageInfo methodsFor: 'accessing' stamp: 'sd 9/12/2010 19:05'!
environment
	^  Smalltalk globals
	! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:20'!
extensionCategoriesForClass: aClass
	^ aClass organization categories select: [:cat | self isYourClassExtension: cat]! !

!PackageInfo methodsFor: 'listing' stamp: 'al 3/1/2006 21:51'!
extensionClasses
	^ self externalBehaviors reject: [:classOrTrait | (self extensionCategoriesForClass: classOrTrait) isEmpty]! !

!PackageInfo methodsFor: 'listing' stamp: 'al 3/1/2006 21:51'!
extensionMethods
	^ self externalBehaviors gather: [:classOrTrait | self extensionMethodsForClass: classOrTrait]! !

!PackageInfo methodsFor: 'testing' stamp: 'avi 4/6/2004 15:16'!
extensionMethodsForClass: aClass
	^ (self extensionCategoriesForClass: aClass)
		gather: [:cat | ((aClass organization listAtCategoryNamed: cat) ifNil: [#()])
							collect: [:sel | self referenceForMethod: sel ofClass: aClass]]! !

!PackageInfo methodsFor: 'testing' stamp: 'dvf 10/18/2002 23:22'!
extensionMethodsFromClasses: classes
	^classes
		gather: [:class | self extensionMethodsForClass: class]! !

!PackageInfo methodsFor: 'modifying' stamp: 'al 3/1/2006 21:42'!
externalBehaviors
	^self externalClasses , self externalTraits! !

!PackageInfo methodsFor: 'dependencies' stamp: 'ab 11/18/2002 01:16'!
externalCallers
	^ self 
		externalRefsSelect: [:literal | literal isKindOf: Symbol] 
		thenCollect: [:l | l].! !

!PackageInfo methodsFor: 'dependencies' stamp: 'ab 6/10/2003 17:18'!
externalClasses
	| myClasses |
	myClasses := self classesAndMetaClasses.
	^ Array streamContents:
		[:s |
		ProtoObject withAllSubclassesDo:
			[:class |
			(myClasses includes: class) ifFalse: [s nextPut: class]]]! !

!PackageInfo methodsFor: 'naming' stamp: 'ab 10/16/2002 21:22'!
externalName
	^ self packageName! !

!PackageInfo methodsFor: 'dependencies' stamp: 'nice 1/5/2010 15:59'!
externalRefsSelect: selBlock thenCollect: colBlock
	| pkgMethods dependents extMethods otherClasses otherMethods classNames |

	classNames := self classes collect: [:c | c name].
	extMethods := self extensionMethods collect: [:mr | mr methodSymbol].
	otherClasses := self externalClasses difference: self externalSubclasses.
	otherMethods :=  otherClasses gather: [:c | c selectors].
	pkgMethods := self methods asSet collect: [:mr | mr methodSymbol].
	pkgMethods removeAllFoundIn: otherMethods.

	dependents := Set new.
	otherClasses do: [:c |
		c selectorsAndMethodsDo:
			[:sel :compiled | | refs |
			(extMethods includes: sel) ifFalse: 
				[refs := compiled literals select: selBlock thenCollect: colBlock.
				refs do: [:ea |
					((classNames includes: ea) or: [pkgMethods includes: ea])
							ifTrue: [dependents add: (self referenceForMethod: sel ofClass: c) -> ea]]]]].
	^ dependents! !

!PackageInfo methodsFor: 'dependencies' stamp: 'stephaneducasse 2/4/2006 20:40'!
externalSubclasses
	| pkgClasses subClasses |
	pkgClasses := self classes.
	subClasses := Set new.
	pkgClasses do: [:c | subClasses addAll: (c allSubclasses)].
	^ subClasses difference: pkgClasses
! !

!PackageInfo methodsFor: 'modifying' stamp: 'nice 1/5/2010 15:59'!
externalTraits
	
	
	^ Array streamContents: [:s | | behaviors |
		behaviors := self classesAndMetaClasses.
		Smalltalk allTraits do: [:trait |
			(behaviors includes: trait) ifFalse: [s nextPut: trait].
			(behaviors includes: trait classSide) ifFalse: [s nextPut: trait classSide]]].			! !

!PackageInfo methodsFor: 'dependencies' stamp: 'ab 11/18/2002 01:15'!
externalUsers
	^ self 
		externalRefsSelect: [:literal | literal isVariableBinding] 
		thenCollect: [:l | l key]! !

!PackageInfo methodsFor: 'accessing' stamp: 'sd 9/12/2010 19:07'!
foreignClasses
	| s |
	s := IdentitySet new.
	self foreignSystemCategories
		do: [ :c | 
			(self systemOrganization listAtCategoryNamed: c)
				do: [ :cl | 
					| cls |
					cls := self environment at: cl.
					s
						add: cls;
						add: cls class ] ].
	^ s ! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:22'!
foreignExtensionCategoriesForClass: aClass
	^ aClass organization categories select: [:cat | self isForeignClassExtension: cat]! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:23'!
foreignExtensionMethodsForClass: aClass
	^ (self foreignExtensionCategoriesForClass: aClass)
		gather: [:cat | (aClass organization listAtCategoryNamed: cat)
						  collect: [:sel | self referenceForMethod: sel ofClass: aClass]]! !

!PackageInfo methodsFor: 'accessing' stamp: 'sd 9/12/2010 19:07'!
foreignSystemCategories
	^ self systemOrganization categories
		reject: [:cat | self includesSystemCategory: cat]  ! !

!PackageInfo methodsFor: 'comparing' stamp: 'avi 10/11/2003 14:20'!
hash
	^ packageName hash! !

!PackageInfo methodsFor: 'testing' stamp: 'StephaneDucasse 2/6/2010 16:39'!
includesChangeRecord: aChangeRecord
	^ aChangeRecord methodClass notNil and:
		[self
			includesMethodCategory: aChangeRecord category
			ofClass: aChangeRecord methodClass]! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:23'!
includesClass: aClass
	^ self includesSystemCategory: aClass theNonMetaClass category! !

!PackageInfo methodsFor: 'testing' stamp: 'sd 9/12/2010 19:07'!
includesClassNamed: aClassName
	^ self includesSystemCategory: ((self systemOrganization categoryOfElement: aClassName) ifNil: [^false])! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 12/5/2002 00:16'!
includesMethod: aSymbol ofClass: aClass
	aClass ifNil: [^ false].
	^ self
		includesMethodCategory: ((aClass organization categoryOfElement: aSymbol)
										ifNil: [' '])
		ofClass: aClass! !

!PackageInfo methodsFor: 'testing' stamp: 'dvf 9/17/2002 00:18'!
includesMethodCategory: categoryName ofClass: aClass
	^ (self isYourClassExtension: categoryName)
		or: [(self includesClass: aClass)
				and: [(self isForeignClassExtension: categoryName) not]]! !

!PackageInfo methodsFor: 'testing' stamp: 'dvf 7/23/2003 14:06'!
includesMethodCategory: categoryName ofClassNamed: aClass
	^ (self isYourClassExtension: categoryName)
		or: [(self includesClassNamed: aClass)
				and: [(self isForeignClassExtension: categoryName) not]]! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/14/2002 18:06'!
includesMethodReference: aMethodRef
	^ self includesMethod: aMethodRef methodSymbol ofClass: aMethodRef actualClass! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:23'!
includesSystemCategory: categoryName
	^ self category: categoryName matches: self systemCategoryPrefix! !

!PackageInfo methodsFor: 'testing' stamp: 'ab 11/13/2002 01:23'!
isForeignClassExtension: categoryName
	^ categoryName first = $* and: [(self isYourClassExtension: categoryName) not]! !

!PackageInfo methodsFor: 'testing' stamp: 'StephaneDucasse 2/6/2010 16:41'!
isOverrideCategory: aString
	^ aString endsWith: '-override'! !

!PackageInfo methodsFor: 'testing' stamp: 'StephaneDucasse 2/6/2010 16:41'!
isOverrideMethod: aMethodReference
	^ self isOverrideCategory: aMethodReference category! !

!PackageInfo methodsFor: 'testing' stamp: 'StephaneDucasse 2/6/2010 16:41'!
isOverrideOfYourMethod: aMethodReference
	"Answers true if the argument overrides a method in this package"
	^ (self isYourClassExtension: aMethodReference category) not and:
		[(self changeRecordForOverriddenMethod: aMethodReference) notNil]! !

!PackageInfo methodsFor: 'testing' stamp: 'avi 3/10/2004 12:37'!
isYourClassExtension: categoryName
	^ categoryName notNil and: [self category: categoryName asLowercase matches: self methodCategoryPrefix]! !

!PackageInfo methodsFor: 'comparing' stamp: 'StephaneDucasse 3/3/2010 13:44'!
linesOfCode
	"An approximate measure of lines of code.
	Includes comments, but excludes blank lines."
	^self methods inject: 0 into: [:sum :each | sum + each compiledMethod linesOfCode]! !

!PackageInfo methodsFor: 'naming' stamp: 'stephaneducasse 2/4/2006 20:40'!
methodCategoryPrefix
	^ methodCategoryPrefix ifNil: [methodCategoryPrefix := '*', self packageName asLowercase]! !

!PackageInfo methodsFor: 'listing' stamp: 'StephaneDucasse 2/6/2010 17:37'!
methods
	^ (self extensionMethods, self coreMethods) select: [:method |
		method isValid
			and: [method isLocalSelector
			and: [method methodSymbol isDoIt not]]]! !

!PackageInfo methodsFor: 'testing' stamp: 'StephaneDucasse 2/6/2010 16:44'!
methodsInCategory: aString ofClass: aClass 
	^Array streamContents: [:stream |
		self methodsInCategory: aString ofClass: aClass 
			do: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'StephaneDucasse 2/6/2010 16:51'!
methodsInCategory: aString ofClass: aClass do: aBlock
	((aClass organization listAtCategoryNamed: aString) ifNil: [^self])
		do: [:sel | aBlock value: (self referenceForMethod: sel ofClass: aClass)]! !

!PackageInfo methodsFor: 'testing' stamp: 'dvf 10/18/2002 23:22'!
outsideClasses
	^ProtoObject withAllSubclasses difference: self classesAndMetaClasses! !

!PackageInfo methodsFor: 'listing' stamp: 'StephaneDucasse 2/6/2010 17:37'!
overriddenMethods
	^ Array streamContents: [:stream |
		self overriddenMethodsDo: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'StephaneDucasse 3/3/2010 13:47'!
overriddenMethodsDo: aBlock
	"Enumerates the methods the receiver contains which have been overridden by other packages"
	^ self allOverriddenMethodsDo: [:ea |
		(self isOverrideOfYourMethod: ea)
			ifTrue: [aBlock value: ea]]! !

!PackageInfo methodsFor: 'listing' stamp: 'StephaneDucasse 2/6/2010 17:38'!
overriddenMethodsInClass: aClass
	^Array streamContents: [:stream |
		self overriddenMethodsInClass: aClass
			do: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'StephaneDucasse 2/6/2010 16:52'!
overriddenMethodsInClass: aClass do: aBlock
	"Evaluates aBlock with the overridden methods in aClass"
	^ self overrideCategoriesForClass: aClass do: [:cat |
		self methodsInCategory: cat ofClass: aClass do: aBlock]! !

!PackageInfo methodsFor: 'testing' stamp: 'StephaneDucasse 2/6/2010 16:45'!
overrideCategoriesForClass: aClass
	^Array streamContents: [:stream |
		self overrideCategoriesForClass: aClass
			do: [:each | stream nextPut: each]]
! !

!PackageInfo methodsFor: 'enumerating' stamp: 'StephaneDucasse 2/6/2010 16:52'!
overrideCategoriesForClass: aClass do: aBlock
	"Evaluates aBlock with all the *foo-override categories in aClass"
	^ aClass organization categories do: [:cat |
		(self isOverrideCategory: cat) ifTrue: [aBlock value: cat]]! !

!PackageInfo methodsFor: 'naming' stamp: 'stephaneducasse 2/4/2006 20:40'!
packageName
	^ packageName ifNil: [packageName := self categoryName]! !

!PackageInfo methodsFor: 'naming' stamp: 'stephaneducasse 2/4/2006 20:40'!
packageName: aString
	packageName := aString! !

!PackageInfo methodsFor: 'printing' stamp: 'stephane.ducasse 8/26/2008 20:43'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '(',self packageName,')'.! !

!PackageInfo methodsFor: 'testing' stamp: 'Janniklaval 10/23/2010 12:22'!
referenceForMethod: aSymbol ofClass: aClass
	^ MethodReference new setClass: aClass methodSymbol: aSymbol! !

!PackageInfo methodsFor: 'registering' stamp: 'avi 11/12/2003 23:12'!
register
	PackageOrganizer default registerPackage: self! !

!PackageInfo methodsFor: 'modifying' stamp: 'avi 10/11/2003 15:14'!
removeMethod: aMethodReference! !

!PackageInfo methodsFor: 'listing' stamp: 'ab 11/14/2002 18:39'!
selectors
	^ self methods collect: [:ea | ea methodSymbol]! !

!PackageInfo methodsFor: 'testing' stamp: 'sd 9/12/2010 19:08'!
systemCategories
	^ self systemOrganization categories select: [:cat | self includesSystemCategory: cat]! !

!PackageInfo methodsFor: 'naming' stamp: 'ab 10/28/2002 10:38'!
systemCategoryPrefix
	^ self packageName! !

!PackageInfo methodsFor: 'testing' stamp: 'sd 9/12/2010 19:08'!
systemOrganization
	^self environment organization! !

Object subclass: #PackageOrganizer
	instanceVariableNames: 'packages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PackageInfo-Base'!

PackageOrganizer class
	instanceVariableNames: 'default'!

PackageOrganizer class
	instanceVariableNames: 'default'!

!PackageOrganizer class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:40'!
default
	^ default ifNil: [default := self new]! !

!PackageOrganizer class methodsFor: 'instance creation' stamp: 'avi 10/13/2003 15:25'!
new
	^ self basicNew initialize! !

!PackageOrganizer methodsFor: 'searching' stamp: 'oha 6/27/2008 15:38'!
allPackagesContainingUnimplementedCalls
	"Answer a Set of Packages that have classes which contain messages that 
	 have unimplemented calls"
	^ (SystemNavigation default allClassesWithUnimplementedCalls keys collect: [:d|
		(self packageOfClass: d) packageName
	]) .
! !

!PackageOrganizer methodsFor: 'cleaning' stamp: 'StephaneDucasse 7/11/2010 22:58'!
flushObsoletePackages: aBlock
	"Flush all packages considered obsolete by evaluating the argument block."

	packages keys do: [ :key |
		(aBlock value: (packages at: key)) ifTrue:[packages removeKey: key].
	].
	self changed: #packages; changed: #packageNames.! !

!PackageOrganizer methodsFor: 'initializing' stamp: 'alain.plantec 5/28/2009 10:13'!
initialize
	super initialize.
	packages := Dictionary new! !

!PackageOrganizer methodsFor: 'private' stamp: 'stephane.ducasse 9/4/2008 12:51'!
mostSpecificPackageIn: aCollection ifNone: aBlock
	aCollection isEmpty
		ifTrue: [ ^ aBlock value ].
	^ (aCollection asArray
		sort: [ :a :b | a packageName size > b packageName size ])
		first! !

!PackageOrganizer methodsFor: 'searching' stamp: 'stephane.ducasse 9/4/2008 12:50'!
mostSpecificPackageOfClass: aClass
	^ self mostSpecificPackageOfClass: aClass ifNone: [self noPackageFound]! !

!PackageOrganizer methodsFor: 'searching' stamp: 'stephane.ducasse 9/4/2008 12:51'!
mostSpecificPackageOfClass: aClass ifNone: aBlock
	
	^ self 
		mostSpecificPackageIn: (self packages select: [ :each | each includesClass: aClass ])
		ifNone: aBlock! !

!PackageOrganizer methodsFor: 'searching' stamp: 'stephane.ducasse 9/4/2008 12:50'!
mostSpecificPackageOfMethod: aMethodReference
	^ self mostSpecificPackageOfMethod: aMethodReference ifNone: [self noPackageFound]! !

!PackageOrganizer methodsFor: 'searching' stamp: 'stephane.ducasse 9/4/2008 12:52'!
mostSpecificPackageOfMethod: aMethodReference ifNone: aBlock

	^ self  
		mostSpecificPackageIn: (self packages select: [ :each | each includesMethodReference: aMethodReference ])
		ifNone: aBlock! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 10/11/2003 14:21'!
noPackageFound
	self error: 'No package found'! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 11/12/2003 23:08'!
packageNamed: aString ifAbsent: errorBlock
	^ packages at: aString ifAbsent: errorBlock! !

!PackageOrganizer methodsFor: 'accessing' stamp: 'avi 11/12/2003 23:01'!
packageNames
	^ packages keys! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 10/11/2003 14:21'!
packageOfClass: aClass
	^ self packageOfClass: aClass ifNone: [self noPackageFound]! !

!PackageOrganizer methodsFor: 'searching' stamp: 'MarcusDenker 3/11/2010 11:28'!
packageOfClass: aClass ifNone: errorBlock
	| classCategory |
	classCategory := aClass theNonMetaClass category.
	packages at: classCategory ifPresent: [:v | ^ v].
	
	(classCategory includes: $-)
		ifTrue: [ packages at: (classCategory copyUpTo: $-) ifPresent: [:v | ^ v] ].
	
	^ self packages detect: [:ea | ea includesClass: aClass] ifNone: errorBlock! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 10/11/2003 14:21'!
packageOfMethod: aMethodReference
	^ self packageOfMethod: aMethodReference ifNone: [self noPackageFound]! !

!PackageOrganizer methodsFor: 'searching' stamp: 'avi 10/11/2003 14:22'!
packageOfMethod: aMethodReference ifNone: errorBlock
	^ self packages detect: [:ea | ea includesMethodReference: aMethodReference] ifNone: errorBlock! !

!PackageOrganizer methodsFor: 'accessing' stamp: 'avi 11/12/2003 23:01'!
packages
	^ packages values! !

!PackageOrganizer methodsFor: 'registering' stamp: 'avi 11/12/2003 23:01'!
registerPackage: aPackageInfo
	packages at: aPackageInfo packageName put: aPackageInfo.
	self changed: #packages; changed: #packageNames.
! !

!PackageOrganizer methodsFor: 'registering' stamp: 'avi 11/12/2003 21:08'!
registerPackageNamed: aString
	^ self registerPackage: (PackageInfo named: aString)! !

!PackageOrganizer methodsFor: 'registering' stamp: 'avi 11/12/2003 23:08'!
unregisterPackage: aPackageInfo
	packages removeKey: aPackageInfo packageName ifAbsent: [].	
	self changed: #packages; changed: #packageNames.
! !

!PackageOrganizer methodsFor: 'registering' stamp: 'avi 11/12/2003 21:10'!
unregisterPackageNamed: aString
	self unregisterPackage: (self packageNamed: aString ifAbsent: [^ self])! !

Object subclass: #PackageServices
	instanceVariableNames: ''
	classVariableNames: 'ServiceClasses'
	poolDictionaries: ''
	category: 'PackageInfo-Base'!

!PackageServices class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 13:01'!
allServices
	^ ServiceClasses gather: [:ea | ea services]! !

!PackageServices class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:40'!
initialize
	ServiceClasses := Set new! !

!PackageServices class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 12:59'!
register: aClass
	ServiceClasses add: aClass! !

!PackageServices class methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 12:59'!
unregister: aClass
	ServiceClasses remove: aClass! !

!PackageServices methodsFor: 'as yet unclassified' stamp: 'avi 10/11/2003 14:06'!
seeClassSide! !
PackageServices initialize!
PackageInfo initialize!
